﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using TimeTrakker;
using Westwind.Utilities;

namespace TimeTrakkerWeb
{
    public partial class NewCustomerOrProject : TimeTrakkerBaseForm
    {

        public busCustomer Customer = null;
        public busProject Project = null;

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            
            Customer = TimeTrakkerFactory.GetCustomer();
            Project = TimeTrakkerFactory.GetProject();

            LoadCustomerList();
        }

        void LoadCustomerList()
        {
            var customerQuery = Customer.GetCustomerList();

            this.lstCustomers.DataSource = Customer.Converter.ToDataReader(customerQuery);
            this.lstCustomers.DataTextField = "Company";
            this.lstCustomers.DataValueField = "Pk";
            this.lstCustomers.DataBind();

            this.lstCustomers.Items.Insert(0, new ListItem() { Text = "--- Please select a customer or leave blank for new", Value = "" });
        }

        protected override void OnLoad(EventArgs e)    
        {
            base.OnLoad(e);

            this.TimeTrakkerMaster.SubTitle = this.Title;
        }

        protected void btnSaveCustomer_Click(object sender, EventArgs e)
        {
            // must load an empty entity to save into
            this.Customer.NewEntity();
            
            this.CustomerDataBinder.Unbind();

            if (this.CustomerDataBinder.BindingErrors.Count > 0)
            {
                this.ErrorDisplay.ShowError(this.CustomerDataBinder.BindingErrors.ToHtml(),
                                            "Please correct the following:");
                return;
            }

            if (!this.Customer.Validate())
            {
                foreach (ValidationError error in this.Customer.ValidationErrors)
                {
                    this.CustomerDataBinder.AddBindingError(error.Message, error.ControlID);
                }
                this.ErrorDisplay.ShowError(this.CustomerDataBinder.BindingErrors.ToHtml(),
                            "Please correct the following:");
                return;
            }

            if (!this.Customer.Save())
            {
                this.ErrorDisplay.ShowError(this.Customer.ErrorMessage);
                return;
            }

            this.ErrorDisplay.ShowMessage("Customer has been saved.");
            
            this.LoadCustomerList();
            this.lstCustomers.SelectedValue = this.Customer.Entity.Pk.ToString();
            
        }

        protected void btnSaveProject_Click(object sender, EventArgs e)
        {            
            this.Project.NewEntity();

            this.ProjectDataBinder.Unbind();

            if (this.ProjectDataBinder.BindingErrors.Count > 0)
            {
                this.ErrorDisplay.ShowError(this.ProjectDataBinder.BindingErrors.ToHtml(),
                                            "Please correct the following:");
                return;
            }

            if (!this.Project.Validate())
            {
                foreach (ValidationError error in this.Project.ValidationErrors)
                {
                    this.ProjectDataBinder.AddBindingError(error.Message, error.ControlID);
                }
                this.ErrorDisplay.ShowError(this.ProjectDataBinder.BindingErrors.ToHtml(),
                            "Please correct the following:");
                return;
            }

            if (!this.Project.Save())
            {
                this.ErrorDisplay.ShowError(this.Project.ErrorMessage);
                return;
            }

            this.ErrorDisplay.ShowMessage("Project has been saved.");
            
        }

        protected void btnSaveCustomerAndProject_Click(object sender, EventArgs e)
        {
            
        }
    }
}