#region License
/*
 **************************************************************
 *  Author: Rick Strahl 
 *           West Wind Technologies, 2008 2011
 *          http://www.west-wind.com/
 * 
 * Created: 09/04/2008
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 **************************************************************  
*/
#endregion


using System;
using System.Web.UI;
using System.IO;
using System.Reflection;
using Westwind.Utilities;

[assembly: TagPrefix("Westwind.Web.Controls", "ww")]

[assembly: WebResource("Westwind.Web.Controls.Resources.jquery.js", "application/x-javascript")]
[assembly: WebResource("Westwind.Web.Controls.Resources.ww.jquery.js", "application/x-javascript")]


[assembly: WebResource("Westwind.Web.Controls.Resources.warning.gif", "image/gif")]
[assembly: WebResource("Westwind.Web.Controls.Resources.info.gif", "image/gif")]
[assembly: WebResource("Westwind.Web.Controls.Resources.loading.gif", "image/gif")]
[assembly: WebResource("Westwind.Web.Controls.Resources.loading_small.gif", "image/gif")]
[assembly: WebResource("Westwind.Web.Controls.Resources.close.gif", "image/gif")]
[assembly: WebResource("Westwind.Web.Controls.Resources.help.gif", "image/gif")]
[assembly: WebResource("Westwind.Web.Controls.Resources.calendar.gif", "image/gif")]



namespace Westwind.Web
{
    /// <summary>
    /// Class is used as to consolidate access to resources
    /// </summary>
    public class ControlResources
    {
        /* Embedded Script Resources */
        public const string JQUERY_SCRIPT_RESOURCE = "Westwind.Web.Controls.Resources.jquery.js";
        public const string WWJQUERY_SCRIPT_RESOURCE = "Westwind.Web.Controls.Resources.ww.jquery.js";

        /*  Icon Resource Strings */
        public const string INFO_ICON_RESOURCE = "Westwind.Web.Controls.Resources.info.gif";
        public const string WARNING_ICON_RESOURCE = "Westwind.Web.Controls.Resources.warning.gif";
        public const string CLOSE_ICON_RESOURCE = "Westwind.Web.Controls.Resources.close.gif";
        public const string HELP_ICON_RESOURCE = "Westwind.Web.Controls.Resources.help.gif";
        public const string LOADING_ICON_RESOURCE = "Westwind.Web.Controls.Resources.loading.gif";
        public const string LOADING_SMALL_ICON_RESOURCE = "Westwind.Web.Controls.Resources.loading_small.gif";
        public const string CALENDAR_ICON_RESOURCE = "Westwind.Web.Controls.Resources.calendar.gif";

        /* Content Types */
        public const string STR_JsonContentType = "application/json";
        public const string STR_JavaScriptContentType = "application/x-javascript";
        public const string STR_UrlEncodedContentType = "application/x-www-form-urlencoded";
        public const string STR_XmlContentType = "text/xml";

         
        /// <summary>
        /// Returns a string resource from a given assembly.
        /// </summary>
        /// <param name="assembly">Assembly reference (ie. typeof(ControlResources).Assembly) </param>
        /// <param name="ResourceName">Name of the resource to retrieve</param>
        /// <returns></returns>
        public static string GetStringResource(Assembly assembly, string ResourceName)
        {
            Stream st = assembly.GetManifestResourceStream(ResourceName);
            StreamReader sr = new StreamReader(st);
            string content = sr.ReadToEnd();
            st.Close();
            return content;
        }


        /// <summary>
        /// Returns a string resource from the from the ControlResources Assembly
        /// </summary>
        /// <param name="ResourceName"></param>
        /// <returns></returns>
        public static string GetStringResource(string ResourceName)
        {
            return GetStringResource(typeof(ControlResources).Assembly, ResourceName);
        }


        #region Obsolete Code
        /// <summary>
        /// Determines what location jQuery is loaded from
        /// </summary>
        [Obsolete("Please use ScriptLoader object instead")]
        public static jQueryLoadModes jQueryLoadMode = jQueryLoadModes.ContentDeliveryNetwork;

        /// <summary>
        /// jQuery CDN Url on Google
        /// </summary>
        [Obsolete("Please use ScriptLoader object instead")]
        public static string jQueryCdnUrl = "//ajax.googleapis.com/ajax/libs/jquery/1.7.1/jquery.min.js";

        /// <summary>
        /// Fallback Url if CDN can't be reached.
        /// </summary>
        [Obsolete("Please use ScriptLoader object instead")]
        public static string jQueryCdnFallbackUrl = "~/scripts/jquery.min.js";

        /// <summary>
        /// jQuery CDN Url on Google
        /// </summary>
        [Obsolete("Please use ScriptLoader object instead")]
        public static string jQueryUiCdnUrl = "//ajax.googleapis.com/ajax/libs/jqueryui/1.8.16/jquery-ui.min.js";

        /// <summary>
        /// jQuery UI fallback Url if CDN is unavailable or WebResource is used
        /// Note: The file needs to exist and hold the minimized version of jQuery ui
        /// </summary>
        [Obsolete("Please use ScriptLoader object instead")]
        public static string jQueryUiLocalFallbackUrl = "~/scripts/jquery-ui.min.js";


        /// <summary>
        /// Inserts a script link to load jQuery into the page based on the jQueryLoadModes settings
        /// of this class. Default load is by CDN plus WebResource fallback
        /// </summary>
        /// <param name="url">
        /// An optional explicit URL to load jQuery from. Url is resolved. 
        /// When specified no fallback is applied
        /// </param>        
        /// <returns>full script tag and fallback script for jQuery to load</returns>
        [Obsolete("Please use ScriptLoader.jQueryLink instead")]
        public static string jQueryLink(jQueryLoadModes jQueryLoadMode = jQueryLoadModes.Default, string url = null)
        {
            return ScriptLoader.jQueryLink(jQueryLoadMode, url);
        }


        /// <summary>
        /// Inserts a script link to load jQuery into the page based on the jQueryLoadModes settings
        /// of this class. Default load is by CDN plus WebResource fallback
        /// </summary>
        /// <param name="url">
        /// An optional explicit URL to load jQuery from. Url is resolved. 
        /// When specified no fallback is applied
        /// </param>        
        /// <returns>full script tag and fallback script for jQuery to load</returns>
        [Obsolete("Please use ScriptLoader.jQueryUiLink instead")]
        public static string jQueryUiLink(jQueryLoadModes jQueryLoadMode = jQueryLoadModes.Default, string url = null)
        {
            return ScriptLoader.jQueryUiLink(jQueryLoadMode, url);
        }


        /// <summary>
        /// Returns a fully qualified script tag for loading ww.jquery.js
        /// </summary>
        /// <param name="jQueryLoadMode"></param>
        /// <param name="url"></param>
        /// <returns></returns>
        [Obsolete("Please use ScriptLoader.wwJQueryLink instead")]
        public static string wwJqueryLink(jQueryLoadModes jQueryLoadMode = jQueryLoadModes.Default, string url = null)
        {
            return ScriptLoader.wwJqueryLink(jQueryLoadMode, url);
        }



        /// <summary>
        /// Loads the appropriate jScript library out of the scripts directory
        /// </summary>
        /// <param name="control"></param>
        /// <param name="jQueryUrl">Optional url to jQuery as a virtual or absolute server path</param>\
        [Obsolete("Please use ScriptLoader.LoadjQuery instead")]
        public static void LoadjQuery(Control control, string jQueryUrl = null)
        {
            ScriptLoader.LoadjQuery(control, jQueryUrl);
        }


        /// <summary>
        /// Loads the ww.jquery.js library from Resources at the end of the Html Header (if available)
        /// </summary>
        /// <param name="control"></param>
        /// <param name="loadjQuery"></param>
        [Obsolete("Please use ScriptLoader.LoadwwjQuery instead")]
        public static void LoadwwjQuery(Control control, bool loadjQuery = true)
        {
            ScriptLoader.LoadwwjQuery(control, loadjQuery);
        }

        /// <summary>
        /// Loads the appropriate jScript library out of the scripts directory
        /// </summary>
        /// <param name="control"></param>
        /// <param name="jQueryUiUrl">Optional url to jQuery as a virtual or absolute server path</param>
        [Obsolete("Please use ScriptLoader.LoadjQueryUi instead")]
        public static void LoadjQueryUi(Control control, string jQueryUiUrl = null)
        {
            ScriptLoader.LoadjQueryUi(control, jQueryUiUrl);
        }
#endregion
    }
}
