﻿using System;
using System.Linq;
using System.Web;
using Westwind.Web;
using System.Text;

namespace Westwind.WebToolkit.Ajax
{
    /// <summary>
    /// Callback Handler for AutoComplete page. Returns stock
    /// information
    /// </summary>
    public class AutoCompleteHandler : CallbackHandler
    {
        /// <summary>
        ///  For jquery.ui AutoComplete Plug in which returns
        ///  an array of label/value objects
        /// </summary>
        /// <returns></returns>
        [CallbackMethod]
        public Array GetStockLookup()
        {
            HttpRequest Request = HttpContext.Current.Request;

            // From Autocomplete plug-in
            string partial = Request.QueryString["term"] as string;

            int maxItems = 50;            
            
            busPortfolioItem stocks = new busPortfolioItem();

           
            
            // filter symbols to search term - note that you must returns label/value, but you can
            // add additional properties which are accessible in the item on the client
            var symbolList = stocks.GetSymbolList(partial)
                                  .Select(stock => new { label = stock.Symbol + " -  " +  stock.Company,
                                                         value = stock.Symbol, 
                                                         exchange = stock.Exchange                                                         
                                  })                                                         
                                  .Take(maxItems);

            return symbolList.ToArray();
        }


        /// <summary>
        /// Returns an individual stock quote from a symbol name
        /// </summary>
        /// <param name="symbol"></param>
        /// <returns></returns>
        [CallbackMethod]
        public StockQuote GetStockQuote(string symbol)
        {
            StockServer server = new StockServer();
            StockQuote quote = server.GetStockQuote(symbol);

            return quote;
        }


        /// <summary>
        /// Returns a stock chart image as a stream. The streamed
        /// data can be accessed from images or other loaded direclty
        /// into a browser.
        /// 
        /// This is useful in order to keep 'service' functionality isolated
        /// to a single service handler rather than offloading non-JSON
        /// requests to another service/handler.
        /// 
        /// Call by Url:
        /// AutocompleteHandler.ashx?Method=GetStockHistoryGraph&symbol=msft
        /// </summary>
        /// <param name="symbol"></param>
        /// <returns></returns>
        [CallbackMethod(ContentType="image/png")]
        public byte[] GetStockHistoryGraph(string symbol)
        {
            StockServer server = new StockServer();
            byte[] img = server.GetStockHistoryGraph(new[] { symbol },
                                                        "History for " + symbol.ToUpper(), 
                                                        450, 300, 2);            
            return img;
        }


        [CallbackMethod(ReturnAsRawString = true, ContentType = "text/plain; charset=UTF8")]
        public string GetAllStockSymbols()
        {
            busPortfolioItem stocks = new busPortfolioItem();
            var symbolList = stocks.GetSymbolList(null)
                                    .Select(stock => new { Company = stock.Company, Symbol = stock.Symbol });

            string fmt = @"new SymbolInfo() {{ Symbol=""{0}"", Company = ""{1}"" }}," + "\r\n";

            StringBuilder sb = new StringBuilder();

            foreach (var symbol in symbolList)
            {
                if (string.IsNullOrEmpty(symbol.Symbol))
                    continue;
                    
                sb.AppendFormat(fmt, symbol.Symbol, symbol.Company);
            }

            return sb.ToString();                                  
        }


    }
}
