﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Westwind.Web;

namespace Westwind.WebToolkit
{
    /// <summary>
    /// Summary description for AlbumService
    /// </summary>
    public class AlbumService : CallbackHandler
    {
        /// <summary>
        /// Retrieve all albums or albums for a specific band by name
        /// </summary>
        /// <param name="band"></param>
        /// <returns></returns>
        [CallbackMethod(RouteUrl="albums/{*band}")]
        public Album[] GetAlbums(string band)
        {
            busAlbum busAlbum = new busAlbum();
            IQueryable<Album> albums;
            if (!string.IsNullOrEmpty(band))
                albums = busAlbum.GetAlbumsByArtist(band);
            else
                albums = busAlbum.GetAllAlbums();
            
            return albums.ToArray();
        }

        /// <summary>
        /// Retrieve an album by album Id
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [CallbackMethod(RouteUrl = "album/{id}")]
        public Album GetAlbum(int id)
        {
            busAlbum busAlbum = new busAlbum();
            Album album = busAlbum.LoadAlbum(id);
            
            if (album == null)
                throw new ArgumentException(busAlbum.ErrorMessage);

            if (!string.IsNullOrEmpty(album.AmazonUrl))
                album.AmazonUrl += "&tag=westwindtechn-20";

            return album;
        }

    }
}