﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Westwind.Web;
using System.IO;
using System.Drawing;
using System.Web.Routing;
using System.Reflection;

namespace Westwind.WebToolkit
{
/// <summary>
/// Summary description for RestService
/// </summary>
// RestService.ashx.cs
public class StockService : CallbackHandler
{

    /// <summary>
    /// Return a single stock quote object
    /// </summary>
    /// <param name="symbol"></param>
    /// <returns></returns>
    [CallbackMethod(RouteUrl = "StockQuote/{symbol}")]
    public StockQuote GetStockQuote(string symbol)
    {
        StockServer server = new StockServer();
        return server.GetStockQuote(symbol);
    }

    /// <summary>
    /// Return an array of stock quotes
    /// </summary>
    /// <param name="symbolList"></param>
    /// <returns></returns>
    [CallbackMethod(RouteUrl = "StockQuotes/{symbolList}")]
    public StockQuote[] GetStockQuotes(string symbolList)
    {
        StockServer server = new StockServer();
        string[] symbols = symbolList.Split(new char[2] { ',', ';' }, StringSplitOptions.RemoveEmptyEntries);
        return server.GetStockQuotes(symbols);
    }

    /// <summary>
    /// Return a raw data result - here a png image is returned as byte[] data
    /// </summary>
    /// <param name="symbolList"></param>
    /// <returns></returns>
    [CallbackMethod(ContentType="image/png",RouteUrl="StockHistory/{symbolList}")]
public byte[] GetStockHistoryGraph(string symbolList)
{
    StockServer server = new StockServer();
    string[] symbols = symbolList.Split(new char[2] { ',', ';' }, StringSplitOptions.RemoveEmptyEntries);
    byte[] imageData = server.GetStockHistoryGraph(symbols, "Stock History", 700, 500, 2);
    return imageData;
}

[CallbackMethod(RouteUrl="StockCompany/{symbol}")]
public string GetCompanyFromStockSymbol(string symbol)
{
    StockServer server = new StockServer();
    StockQuote quote = server.GetStockQuote(symbol);
    return quote.Company;
}

}
}
