﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.Entity;
using System.ComponentModel.DataAnnotations;

namespace Westwind.WebToolkit
{
    public class Album
    {   [Key]
        public int Id {get; set;}
        
        [MaxLength(256)]
        [Required]
        public string AlbumName {get; set; }
        [MaxLength(80)]
        public string Label {get; set; }
        public int? YearReleased {get; set; }

        [MaxLength(256)]
        public string AlbumImageUrl { get; set; }

        [MaxLength(300)]
        public string AmazonUrl { get; set; }
        
        public virtual ICollection<Song> Songs {get; set;} 
        public virtual Artist Artist {get; set;} 

        public Album()
        {
            Songs = new HashSet<Song>();
        }
    }

    public class Song
    {
        [Key]
        public int Id {get; set; }

        [MaxLength(256)]
        [Required]
        public string SongName {get; set; }
        [MaxLength(256)]
        public string Composer {get; set; }
        [MaxLength(6)]
        public string SongLength { get; set; }

        internal virtual Artist Artist {get; set; }
        internal virtual Album Album {get; set; }
    }

    public class Artist
    {
        [Key]
        public int Id {get; set; }
        
        [MaxLength(80)]
        [Required]
        public string ArtistName { get; set; }
        public string Location {get; set;}
        public int? YearFounded {get; set; }

        internal ICollection<Album> Albums { get; set; }

        public Artist()
        {
            Albums = new HashSet<Album>();
        }
    }

    public class AlbumContext : DbContext 
    {

        public AlbumContext()
            : base("name=AlbumContext")
        {
        }

        public AlbumContext(string connectionName) : base("name=" + connectionName)
        {
        }

        public DbSet<Artist> Artists {get; set; }
        public DbSet<Album> Albums {get; set; }
        public DbSet<Song> Songs {get; set; }        
    }

    public class AlbumContextDbInitializer : DropCreateDatabaseAlways<AlbumContext>    
        //DropCreateDatabaseIfModelChanges<AlbumContext>
        
    {
        protected override void Seed(AlbumContext context)
        {
            // Set extra integrity rules 
            //SetDeleteAndUpdateRules(context);

            // Seed initial data
            SeedInitialData(context);
        }

        private void SeedInitialData(AlbumContext context)
        {
            var busAlbum = new busAlbum();
            
            // must used the supplied context or else
            // requests won't work
            busAlbum.Context = context;
            
            busAlbum.NewEntity("The End of the Silence","Rollins Band");
            busAlbum.Entity.AlbumImageUrl = "http://ecx.images-amazon.com/images/I/51FO3rb1tuL._SL160_AA160_.jpg";
            busAlbum.Entity.AmazonUrl = "http://www.amazon.com/End-Silence-Rollins-Band/dp/B0000040OX/ref=sr_1_5?ie=UTF8&qid=1302232195&sr=8-5";
        
            busAlbum.Entity.Artist.Location = "Los Angeles";
            busAlbum.Entity.Artist.YearFounded = 1985;

            busAlbum.AddSong("Low Self Opinion", "Rollins Band", "5:24");
            busAlbum.AddSong("Grip", "Rollins Band", "4:51");
            busAlbum.AddSong("Tearing", "Rollins Band", "4:56");
            busAlbum.AddSong("You didn't need", "Rollins Band", "5:30");
            busAlbum.AddSong("Almost Real", "Rollins Band");
            busAlbum.AddSong("Obscene", "Rollins Band");
            busAlbum.AddSong("What do you do", "Rollins Band");
            busAlbum.AddSong("Another Life", "Rollins Band");
            busAlbum.AddSong("Just like You", "Rollins Band");

            busAlbum.Save();

            busAlbum.NewEntity("Weight", "Rollins Band");
            busAlbum.Entity.AlbumImageUrl = "http://ecx.images-amazon.com/images/I/41eHEGu8NML._AA115_.jpg";
            busAlbum.Entity.AmazonUrl = "http://www.amazon.com/Weight-Rollins-Band/dp/B0000040P3/ref=sr_1_1?ie=UTF8&s=music&qid=1302232341&sr=8-1";
            busAlbum.AddSong("Disconnect", "Rollins Band");
            busAlbum.AddSong("Fool", "Rollins Band");
            busAlbum.AddSong("Civilized", "Rollins Band");
            busAlbum.AddSong("Divine", "Rollins Band");
            busAlbum.AddSong("Liar", "Rollins Band");
            busAlbum.AddSong("Step Back", "Rollins Band");
            busAlbum.AddSong("Wrong Man", "Rollins Band");
            busAlbum.AddSong("Volume 4", "Rollins Band");
            busAlbum.AddSong("Tired", "Rollins Band");
            busAlbum.AddSong("Alien Blueprint", "Rollins Band");
            busAlbum.AddSong("Shine", "Rollins Band");

            busAlbum.NewEntity("Come in and Burn", "Rollins Band");
            busAlbum.Entity.AlbumImageUrl = "http://ecx.images-amazon.com/images/I/41SN5JT2TPL._SL160_AA160_.jpg";
            busAlbum.Entity.AmazonUrl = "http://www.amazon.com/Come-Burn-Rollins-Band/dp/B000005ALZ/ref=sr_1_1?s=music&ie=UTF8&qid=1302232536&sr=1-1";
            busAlbum.AddSong("Shame", "Rollins Band");
            busAlbum.AddSong("Starve", "Rollins Band");
            busAlbum.AddSong("All I want", "Rollins Band");
            busAlbum.AddSong("The End of Something", "Rollins Band");
            busAlbum.AddSong("On my Way to the Cage", "Rollins Band");
            busAlbum.AddSong("Thursday Afternoon", "Rollins Band");
            busAlbum.AddSong("During a City", "Rollins Band");
            busAlbum.AddSong("Neon", "Rollins Band");
            busAlbum.AddSong("Spilling over the Side", "Rollins Band");
            busAlbum.AddSong("Inhale Exhale", "Rollins Band");
            busAlbum.AddSong("Saying Goodbye Again", "Rollins Band");
            busAlbum.AddSong("Rejection", "Rollins Band");


            busAlbum.NewEntity("Dirty Deeds Done Dirt Cheap", "AC/DC");
            busAlbum.Entity.AlbumImageUrl = "http://ecx.images-amazon.com/images/I/61kTaH-uZBL._AA115_.jpg";
            busAlbum.Entity.AmazonUrl = "http://www.amazon.com/Dirty-Deeds-Done-Dirt-Cheap/dp/B00008BXJ4/ref=sr_1_1?s=music&ie=UTF8&qid=1302232491&sr=1-1";
            busAlbum.Entity.Artist.YearFounded = 1973;
            busAlbum.Entity.Artist.Location = "Australia";

            busAlbum.AddSong("Dirty Deeds Done Dirt Cheap", "Angus Young/Bon Scott/Malcolm Young","4:11");
            busAlbum.AddSong("Love at First Feel", "Angus Young/Bon Scott/Malcolm Young","3:10");
            busAlbum.AddSong("Big Balls", "Angus Young/Bon Scott/Malcolm Young","2:38");
            busAlbum.AddSong("Rocker", "Angus Young/Bon Scott/Malcolm Young","2:49");
            busAlbum.AddSong("Problem Child", "Angus Young/Bon Scott/Malcolm Young","5:44");
            busAlbum.AddSong("There's going to be some Rocking", "Angus Young/Bon Scott/Malcolm Young","3:17");
            busAlbum.AddSong("Ain't no Fun (waiting around to be a millionaire)", "Angus Young/Bon Scott/Malcolm Young","7:29");
            busAlbum.AddSong("Ride On", "Angus Young/Bon Scott/Malcolm Young","5:49");
            busAlbum.AddSong("Squealer", "Angus Young/Bon Scott/Malcolm Young","5:14");


            busAlbum.NewEntity("Echoes, Silence, Patience & Grace", "Foo Fighters");
            busAlbum.Entity.AlbumImageUrl = "http://ecx.images-amazon.com/images/I/41grMp2g0RL._SL160_AA160_.jpg";
            busAlbum.Entity.AmazonUrl = "http://www.amazon.com/Echoes-Silence-Patience-Grace-Fighters/dp/B000UFAURI/ref=sr_1_1?s=music&ie=UTF8&qid=1302232599&sr=1-1";
            busAlbum.Entity.Artist.Location = "Seattle";
            busAlbum.Entity.Artist.YearFounded = 1994;

            busAlbum.AddSong("The Pretender", "Foo Fighters");
            busAlbum.AddSong("Let it Die", "Foo Fighters");
            busAlbum.AddSong("Erase/Replace", "Foo Fighters");
            busAlbum.AddSong("Long Road to Ruin", "Foo Fighters");
            busAlbum.AddSong("Come Alive", "Foo Fighters");
            busAlbum.AddSong("Stranger Things have Happened", "Foo Fighters");
            busAlbum.AddSong("Cheer up, Boys", "Foo Fighters");
            busAlbum.AddSong("Summer's End", "Foo Fighters");
            busAlbum.AddSong("Ballad of BeaconsField Miners", "Foo Fighters");
            busAlbum.AddSong("Statues", "Foo Fighters");
            busAlbum.AddSong("But Honestly", "Foo Fighters");
            busAlbum.AddSong("Home", "Foo Fighters");            
        }
    }
        
}